from django.contrib.admin.widgets import AdminFileWidget
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe
from easy_thumbnails.files import get_thumbnailer


class AdminImageWidget(AdminFileWidget):

    def render(self, name, value, attrs=None):
        output = []
        if value and getattr(value, 'url', None):
            options = {'size': (140, 140), 'crop': False}
            image_url = value.url
            thumb_url = get_thumbnailer(value).get_thumbnail(options).url
            file_name = str(value)

            output.append(
                u' <a href="%s" target="_blank"><img src="%s" alt="%s" style="height: 100px;" /></a><br /> %s ' %
                (image_url, thumb_url, file_name, _('Change:')))
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _
from adminsortable.admin import SortableAdmin

from .models import SlideShow as SlideShowModel
from .admin import ImageAdminInline as Image


class SlideShowPlugin(CMSPluginBase, SortableAdmin):
    model = SlideShowModel
    name = _('Slideshow')
    render_template = 'slideshow.html'
    inlines = [Image, ]

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context

plugin_pool.register_plugin(SlideShowPlugin)


from django.contrib.admin.sites import AdminSite
from django.test import TestCase

# Create your tests here.

from .models import SlideShow, Image
from .admin import ImageAdminInline
from .widgets import AdminImageWidget


class MockRequest(object):
    pass


class MockSuperUser(object):
    def has_perm(self, perm):
        return True


class SlideShowPluginTests(TestCase):
    def setUp(self):
        from .cms_plugins import SlideShowPlugin
        self.plugin = SlideShowPlugin()
        self.site = AdminSite()
        self.request = MockRequest()
        self.request.user = MockSuperUser()

        self.test_slideshow = SlideShow()
        self.test_slideshow.save()

        self.test_image = Image(
            name='bla',
            slideshow_id=self.test_slideshow.pk
        )
        self.test_image.save()

    def test_without_image(self):
        """
        test wether the plugin/model instance is in the context
        and the instance contains no images
        """
        instance = SlideShow()

        rendered_html_dict = self.plugin.render({}, instance, None)

        self.assertIn('instance', rendered_html_dict)
        self.assertEqual(rendered_html_dict['instance'].images.all().count(), 0)

    def test_with_image(self):
        """
        test wether the plugin/model instance is in the context
        and the instance contains the right number of images
        """
        instance = self.test_slideshow
        rendered_html_dict = self.plugin.render({}, instance, None)

        self.assertIn('instance', rendered_html_dict)
        self.assertEqual(rendered_html_dict['instance'].images.all().count(), 1)

    def test_admin(self):
        """
        test if the admin site contains the right fields
        """
        image_admin = ImageAdminInline(SlideShow, self.site)
        self.assertEqual(image_admin.get_fieldsets(self.request),
                         [(None, {'fields': ['slideshow', 'name', 'image']})])
        self.assertEqual(image_admin.get_fieldsets(self.request, self.test_image),
                         [(None, {'fields': ['slideshow', 'name', 'image']})])

    def test_admin_widget(self):
        """
        test if the admin site displays a custom image widget
        """
        image_admin = ImageAdminInline(SlideShow, self.site)
        dbfields = image_admin.model._meta.fields

        for dbfield in dbfields:
            if dbfield.name == 'image':
                self.assertTrue(type(image_admin.formfield_for_dbfield(dbfield).widget) is AdminImageWidget)


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Image.url'
        db.add_column(u'slideshow_image', 'url',
                      self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True),
                      keep_default=False)


        # Changing field 'Image.name'
        db.alter_column(u'slideshow_image', 'name', self.gf('django.db.models.fields.CharField')(max_length=255))

    def backwards(self, orm):
        # Deleting field 'Image.url'
        db.delete_column(u'slideshow_image', 'url')


        # Changing field 'Image.name'
        db.alter_column(u'slideshow_image', 'name', self.gf('django.db.models.fields.CharField')(max_length=50))

    models = {
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        u'slideshow.image': {
            'Meta': {'ordering': "['order']", 'object_name': 'Image'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'slideshow': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'images'", 'to': u"orm['slideshow.SlideShow']"}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'slideshow.slideshow': {
            'Meta': {'ordering': "['order']", 'object_name': 'SlideShow', '_ormbases': ['cms.CMSPlugin']},
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'})
        }
    }

    complete_apps = ['slideshow']


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Image'
        db.create_table('slideshow_image', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100)),
            ('slideshow', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['slideshow.SlideShow'], related_name='images')),
        ))
        db.send_create_signal('slideshow', ['Image'])

        # Adding model 'SlideShow'
        db.create_table('slideshow_slideshow', (
            ('cmsplugin_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['cms.CMSPlugin'], primary_key=True, unique=True)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True)),
        ))
        db.send_create_signal('slideshow', ['SlideShow'])

    def backwards(self, orm):
        # Deleting model 'Image'
        db.delete_table('slideshow_image')

        # Deleting model 'SlideShow'
        db.delete_table('slideshow_slideshow')


    models = {
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'auto_now': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'to': "orm['cms.CMSPlugin']", 'null': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'blank': 'True', 'null': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'slideshow.image': {
            'Meta': {'ordering': "['order']", 'object_name': 'Image'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'slideshow': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['slideshow.SlideShow']", 'related_name': "'images'"})
        },
        'slideshow.slideshow': {
            'Meta': {'ordering': "['order']", 'object_name': 'SlideShow', '_ormbases': ['cms.CMSPlugin']},
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'primary_key': 'True', 'unique': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'})
        }
    }

    complete_apps = ['slideshow']


from django.contrib import admin
from adminsortable.admin import SortableAdmin, SortableTabularInline

from .widgets import AdminImageWidget
from .models import SlideShow, Image

# Register your models here.


class ImageAdminInline(SortableTabularInline):
    model = Image

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'image':
            kwargs.pop('request', None)
            kwargs['widget'] = AdminImageWidget
            return db_field.formfield(**kwargs)
        return super(ImageAdminInline, self).formfield_for_dbfield(db_field, **kwargs)


class SlideShowAdmin(SortableAdmin):
    inlines = [ImageAdminInline, ]

admin.site.register(SlideShow, SlideShowAdmin)


#!/usr/bin/env python
import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

INSTALL_REQUIRES = [
    "django-admin-sortable==1.6.7",
]

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Natural Language :: Dutch",
    "Natural Language :: English",
    "Programming Language :: Python :: 2.7",
    "Framework :: Django",
    "Intended Audience :: Developers",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Multimedia :: Graphics :: Viewers",
    "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    "Operating System :: OS Independent",
]

setup(
    name = "djangocms-slideshow",
    version = "0.1.3",
    author = "Urga",
    author_email = "dries@urga.be",
    description = "A simple django cms slideshow plugin",
    license = "GNU",
    keywords = ["slideshow", "django", "cms", "plugin"],
    url = "https://github.com/urga/djangocms-slideshow",
    packages=['slideshow', 'slideshow.migrations', ],
    install_requires=INSTALL_REQUIRES,
    classifiers=CLASSIFIERS,
    long_description=read('README.md'),
    include_package_data=True,
    zip_safe=False
)


from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.utils.translation import ugettext_lazy as _
from adminsortable.models import Sortable

# Create your models here.


class Image(Sortable):
    slideshow = models.ForeignKey('SlideShow', verbose_name=_('Slideshow'), related_name=_('images'))
    name = models.CharField(_('Name'), max_length=255)
    image = models.ImageField(_('Image'), upload_to='slideshow')
    url = models.CharField(_('Url'), max_length=255, blank=True, null=True)

    class Meta(Sortable.Meta):
        verbose_name = _("Image")
        verbose_name_plural = _("Images")
        pass


class SlideShow(CMSPlugin, Sortable):

    def copy_relations(self, old_instance):
        for img in old_instance.images.all():
            img.pk = None
            img.slideshow = self
            img.save()

    class Meta(Sortable.Meta):
        verbose_name = _("Slideshow")
        verbose_name_plural = _("Slideshows")
        pass